import 'dart:convert';

import 'package:yona_crypto/b64.dart';
import 'package:yona_crypto/crypto.dart' as crypto;

import 'peer.dart';

BigInt genMsgID() {
  return BigInt.from(DateTime.now().microsecondsSinceEpoch);
}

class Msg {
  final BigInt id;
  final String data;
  final Peer from;

  Msg(this.id, this.data, this.from);

  static Msg fromString(String str) {
    var data = Uri.splitQueryString(str);
    return Msg(BigInt.parse(data["id"]!), data["data"]!,
        Peer.fromString(data["from"]!));
  }

  @override
  String toString() {
    return Uri(queryParameters: {
      "id": id.toString(),
      "data": data.toString(),
      "from": from.toString(),
    }).query;
  }
}

class IOMsg {
  final B64 from;
  final B64 to;
  final String data;
  final BigInt? id;

  IOMsg(this.from, this.to, this.data, [this.id]);

  static IOMsg fromString(String string, [B64? to]) {
    var data = Uri.splitQueryString(string);
    if (data["to"] != null) {
      to = B64(data["to"]!);
    }
    return IOMsg(B64(data["from"]!), to!, data["data"]!,
        BigInt.tryParse(data["id"] ?? ""));
  }

  static Future<IOMsg> fromMsg(Msg msg, B64 senderSK, B64 receiverPK) async {
    var key = await crypto.deriveKey(senderSK, receiverPK);
    var data = await crypto.encrypt(key, utf8.encode(msg.toString()));
    return IOMsg(crypto.derivePK(senderSK), receiverPK, data);
  }

  Future<Msg> decrypt(B64 toSK) async {
    if (!(crypto.derivePK(toSK) == to)) {
      throw Exception("this msg does not belongs to you");
    }
    var key = await crypto.deriveKey(toSK, from);
    var decrypted = utf8.decode(await crypto.decrypt(key, data));
    return Msg.fromString(decrypted);
  }

  @override
  String toString() {
    return Uri(queryParameters: {
      "from": from.toString(),
      "to": to.toString(),
      "data": data.toString(),
    }).query;
  }
}
