import 'dart:convert';
import 'dart:io';

import 'package:yona/src/addr.dart';
import 'package:yona/src/sse.dart' as sse;

class _SSE implements sse.SSE {
  final Socket _socket;
  late final Stream<String> _stream;

  _SSE(this._socket) {
    _stream = _socket
        .map((a) => a.toList())
        .transform(Utf8Decoder())
        .transform(LineSplitter())
        .where((line) => line.startsWith("data:"))
        .map((line) => line.substring("data:".length).trim())
        .where((data) => data.isNotEmpty);
  }

  @override
  void close() {
    return _socket.destroy();
  }

  @override
  Stream<String> get stream => _stream;
}

Future<sse.SSE> connect(NetAddr addr) async {
  var req = await HttpClient().get(addr.host, addr.port, "/");
  req.headers.set('Accept', 'text/event-stream');
  var rep = await req.close();
  if (rep.statusCode != 200) {
    throw Exception(rep.statusCode);
  }
  return _SSE(await rep.detachSocket());
}

Future<int> send(NetAddr addr, String id, String data) async {
  var socket = await Socket.connect(addr.host, addr.port);
  try {
    var len = data.length;
    socket.write("POST /$id HTTP/1.0\r\nContent-Length: $len\r\n\r\n");
    socket.write(data);
    await socket.flush();
    var line = await socket
        .map((event) => event.toList())
        .transform(Utf8Decoder())
        .transform(LineSplitter())
        .first;
    return int.parse(line.split(" ")[1]);
  } finally {
    socket.destroy();
  }
}
