import 'dart:async';
import 'dart:html' as web;

import 'package:http/http.dart' as http;

import 'addr.dart';
import 'sse.dart' as sse;

class _SSE implements sse.SSE {
  final web.EventSource _es;
  late final Stream<String> _queue;

  _SSE(this._es) {
    _queue = _es.onMessage
        .where((event) => event.data.isNotEmpty)
        .map((event) => event.data.toString())
        .asBroadcastStream();
  }

  @override
  void close() {
    return _es.close();
  }

  @override
  // TODO: implement stream
  Stream<String> get stream => _queue;
}

Future<sse.SSE> connect(NetAddr addr) async {
  return _SSE(web.EventSource(
      Uri(scheme: "http", host: addr.host, port: addr.port).toString()));
}

Future<int> send(NetAddr addr, String id, String data) async {
  var resp = await http.post(
      Uri(scheme: "http", host: addr.host, port: addr.port, pathSegments: [id]),
      body: data);
  return resp.statusCode;
}
